'use strict';

var Quill = window.Quill = require('quill');
Quill.register(Quill.import('attributors/style/align'), true);
Quill.register(Quill.import('attributors/style/direction'), true);

var size = Quill.import('attributors/style/size');
size.whitelist = ['10px', '12px', '14px', '16px', '18px', '20px', '22px', '24px', '26px', '28px', '30px', '32px', '34px', '36px', '38px', '40px', '42px', '44px', '46px', '48px', '50px', '52px', '54px', '56px', '58px', '60px', '62px', '64px', '66px', '68px', '70px', '72px'];
Quill.register(size, true);

var font = Quill.import('attributors/style/font');
font.whitelist = ['arial', 'arial-black', 'helvetica', 'trebuchet-ms', 'verdana', 'tahoma', 'georgia', 'times-new-roman', 'lucida-bright', 'palatino', 'courier'];
Quill.register(font, true);

var quillEditor = {
  install: function install(Vue) {
    var getInstanceName = function getInstanceName(el, binding, vnode) {
      var customInstanceName = '';
      if (binding.arg) {
        customInstanceName = binding.arg;
      } else if (vnode.data.attrs && vnode.data.attrs.instanceName) {
        customInstanceName = vnode.data.attrs.instanceName;
      } else if (el.id) {
        customInstanceName = el.id;
      }
      var instanceName = customInstanceName || 'quill';
      return instanceName;
    };
    Vue.directive('quill', {
      inserted: function inserted(el, binding, vnode) {
        var _this = vnode.context;
        var instanceName = getInstanceName(el, binding, vnode);
        var options = binding.value || {};
        var quill = _this[instanceName];
        var eventEmit = function eventEmit(vnode, name, data) {
          var handlers = vnode.data && vnode.data.on || vnode.componentOptions && vnode.componentOptions.listeners;
          if (handlers && handlers[name]) {
            handlers[name].fns(data);
          }
        };
        if (!quill) {
          var defaultOptions = {
            modules: {
              toolbar: [[{ 'font': [false, 'arial', 'arial-black', 'helvetica', 'trebuchet-ms', 'verdana', 'tahoma', 'georgia', 'times-new-roman', 'lucida-bright', 'palatino', 'courier'] }], [{ 'size': ['10px', '12px', '14px', false, '18px', '20px', '22px', '24px', '26px', '28px', '30px', '32px', '34px', '36px', '38px', '40px', '42px', '44px', '46px', '48px', '50px', '52px', '54px', '56px', '58px', '60px', '62px', '64px', '66px', '68px', '70px', '72px'] }], ['bold', 'italic', 'underline', 'strike'], ['blockquote', 'code-block'], [{ 'header': 1 }, { 'header': 2 }], [{ 'list': 'ordered' }, { 'list': 'bullet' }], [{ 'script': 'sub' }, { 'script': 'super' }], [{ 'indent': '-1' }, { 'indent': '+1' }], [{ 'direction': 'rtl' }], [{ 'header': [1, 2, 3, 4, 5, 6, false] }], [{ 'color': [] }, { 'background': [] }], [{ 'align': [] }], ['clean'], ['link', 'image', 'video']]
            },
            placeholder: 'Insert text here ...',
            readOnly: false,
            theme: 'snow',
            boundary: document.body
          };
          options.theme = options.theme || defaultOptions.theme;
          options.modules = options.modules || defaultOptions.modules;
          options.readOnly = options.readOnly || defaultOptions.readOnly;
          options.boundary = options.boundary || defaultOptions.boundary;
          options.placeholder = options.placeholder || defaultOptions.placeholder;
          options.modules.toolbar = options.modules.toolbar !== undefined ? options.modules.toolbar : defaultOptions.modules.toolbar;
          quill = _this[instanceName] = new Quill(el, options);

          var model = vnode.data.model;
          var content = vnode.data.attrs ? vnode.data.attrs.content : null;

          if (model || content) {
            quill.pasteHTML(model ? model.value : content);
          }

          quill.on('selection-change', function (range) {
            if (!range) {
              eventEmit(vnode, 'blur', quill);
            } else {
              eventEmit(vnode, 'focus', quill);
            }
          });

          quill.on('text-change', function (delta, oldDelta, source) {
            var html = el.children[0].innerHTML;
            var text = quill.getText();
            if (html === '<p><br></p>') {
              html = '';
              quill.root.innerHTML = html;
            }
            if (model) {
              model.callback(html);
            }
            eventEmit(vnode, 'change', {
              editor: quill,
              html: html,
              text: text
            });
          });

          eventEmit(vnode, 'ready', quill);
        }
      },
      componentUpdated: function componentUpdated(el, binding, vnode) {
        var _this = vnode.context;
        var instanceName = getInstanceName(el, binding, vnode);
        var options = binding.value || {};
        var quill = _this[instanceName];
        if (quill) {
          var model = vnode.data.model;
          var content = vnode.data.attrs ? vnode.data.attrs.content : null;
          var newData = model ? model.value : content;
          var oldData = el.children[0].innerHTML;
          if (newData) {
            if (newData != oldData) {
              var range = quill.getSelection();
              quill.root.innerHTML = newData;
              setTimeout(function () {
                quill.setSelection(range);
              });
            }
          } else {
            quill.setText('');
          }
        }
      },
      unbind: function unbind(el, binding, vnode) {
        if (vnode.context[binding.arg]) {
          vnode.context[binding.arg] = null;
          delete vnode.context[binding.arg];
        }
      }
    });
  }
};

module.exports = quillEditor;
